# -*- coding: utf-8 -*-
"""
    eve-swagger
    ~~~~~~~~~~~
    swagger.io extension for Eve-powered REST APIs.

    :copyright: (c) 2015 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""
from .swagger import get_swagger_blueprint, add_documentation  # noqa
from .definitions import INFO, HOST  # noqa
